﻿using System;
using System.Text;

using Autodesk.AutoCAD.ApplicationServices;
using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.GraphicsInterface;
using Autodesk.AutoCAD.Runtime;

[assembly: CommandClass(typeof(RSNNAcadApp.DimensionCommands.Hochzahlen))]

namespace RSNNAcadApp.DimensionCommands
{
    internal class DimOverrule : DrawableOverrule
    {

        public DimOverrule()
        {
            Overrule.Overruling = true;  // if you don't do this, nothing works
        }

        public override bool WorldDraw(Drawable drawable, WorldDraw wd)
        {
            bool result = true;
            Dimension dim = drawable as Dimension;
            if (dim != null)
            {
                using (Dimension clone = (Dimension)dim.Clone())
                {
                    clone.DimensionText = Hochzahlen.GetDimTextExponent(clone);
                    //clone.Prefix = "Prefix Test ";

                    result &= base.WorldDraw(clone, wd);
                }
            }
            else
                result &= base.WorldDraw(drawable, wd);
            return result;
        }
    }


    internal class CircleOverrule : DrawableOverrule
    {

        public CircleOverrule()
        {
            Overrule.Overruling = true;  // if you don't do this, nothing works
        }

        public override bool WorldDraw(Drawable drawable, WorldDraw wd)
        {
            bool result = true;
            Circle dim = drawable as Circle;
            if (dim != null)
            {
                using (Circle clone = (Circle)dim.Clone())
                {
                    //clone.DimensionText = ChangeStyle.GetDimTextExponent(clone);
                    clone.Radius = 10;

                    result &= base.WorldDraw(clone, wd);
                }
            }
            else
                result &= base.WorldDraw(drawable, wd);
            return result;
        }
    }


    class Hochzahlen
    {
        private static CircleOverrule myCircOverrule = null;

        [CommandMethod("CircOVERRULE")]
        public static void OverruleCommand()
        {
            if (myCircOverrule == null)
            {
                myCircOverrule = new CircleOverrule();
                Overrule.AddOverrule(RXObject.GetClass(typeof(Circle)), myCircOverrule, false);
                //Overrule.Overruling = true;
            }
            else
            {
                // Toggle Overruling on/off
                Overrule.Overruling ^= true;
            }

            Application.DocumentManager.MdiActiveDocument.Editor.Regen();
        }

     
     
        private static DimOverrule myDimOverrule = null;

        [CommandMethod("DimOVERRULE")]
        public static void DimOverruleCommand()
        {
            if (myDimOverrule == null)
            {
                myDimOverrule = new DimOverrule();
                Overrule.AddOverrule(RXObject.GetClass(typeof(Dimension)), myDimOverrule, false);
                //Overrule.Overruling = true;
            }
            else
            {
                // Toggle Overruling on/off
                Overrule.Overruling ^= true;
            }

            Application.DocumentManager.MdiActiveDocument.Editor.Regen();
        }


        public static string GetDimTextExponent(Dimension dim)
        {
            Type dimType = dim.GetType();

            // Es werden nur Längenmaße überschrieben
            if ((dimType == typeof(LineAngularDimension2)) | (dimType == typeof(OrdinateDimension)) | (dimType == typeof(Point3AngularDimension)))
                return null;

            // Wenn keine Nachkommastellen vorhanden sind, können wir auch abbrechen
            if (dim.Dimdec == 0)
                return null;

            string dimText = dim.DimensionText;

            // Ist der Bemaßungstext überschrieben und beinhaltet kein Maß, können wir abbrechen
            if (dimText != "")
            {
                if (!dimText.Contains("<>"))
                    return null;
            }


            double dimMeasurement = dim.Measurement;
            double dimRound = dim.Dimrnd;
            int dimDec = dim.Dimdec;

            if (dimRound != 0)
                //dimMeasurement = Utils.MyUtils.RoundedValue(dimMeasurement, dimRound);
               dimMeasurement = Math.Floor((dimMeasurement + dimRound / 2) / dimRound) * dimRound;

            // Wir setzten den Text für die Bemaßungslänge
            string dimTextMeasurement = dimMeasurement.ToString("F" + dimDec.ToString());

            // Wenn die Zeichnung in Metern un der Dimsclae 1 ist, das Maß ebenfalls kleiner als 1 ist, ...
            if (HostApplicationServices.WorkingDatabase.Insunits == UnitsValue.Meters & dim.Dimscale == 1 & dimMeasurement < 1)
            {
                // ... wandeln wir in cm um und reduzieren die Nachkommastellen.
                dimDec -= 2;
                dimTextMeasurement = (dimMeasurement * 100).ToString("F" + dimDec.ToString());
            }

            StringBuilder sbDim = new StringBuilder(dimTextMeasurement);

            if (dimTextMeasurement.EndsWith("0"))
            {
                if (dimDec < 2)
                    sbDim.Remove(sbDim.Length - 2, 2);
                else
                    sbDim.Remove(sbDim.Length - 1, 1);
            }
            else
            {
                if (dimDec < 2)
                    sbDim.Remove(sbDim.Length - 2, 1);

                sbDim.Insert(sbDim.Length - 1, @"{\A1;\H0.7x;\S");
                sbDim.Insert(sbDim.Length, @"^;}");
            }

            if (dim.Prefix != "")
                sbDim.Insert(0, dim.Prefix);
            if (dim.Suffix != "")
                sbDim.Insert(sbDim.Length, dim.Suffix);
            if (dimText != "")
            {
                int Wo = dimText.IndexOf("<>");
                if (Wo > 0)
                    sbDim.Insert(0, dimText.Substring(0, Wo));
                if (Wo < dimText.Length - 2)
                    sbDim.Insert(sbDim.Length, dimText.Substring(Wo + 2));
            }


            return sbDim.ToString();
        }

    }
}